#include <stdio.h>
#include <stdlib.h>

#include "unanm.h"
#include "lzw.h"
#include "fileio.h"

/*

This code is copyright 2004 by Michael Kohn
You may not use any of this program in your own programs
without permission from Michael Kohn <mike@mikekohn.net>

*/

int main(int argc, char *argv[])
{
FILE *in,*out;
struct anm_header_t anm_header;
int palette[256];
char filename[256];
int r,g,b,t;
int *picture;
int total_bytes;
int count;
int x,y;
int width,height;
int max_color;
float i;

  in=fopen(argv[1],"rb");
  if (in==0)
  {
    printf("Couldn't open input file: %s\n",argv[1]);
    exit(1);
  }

  read_chars(in,anm_header.signature,12);
  anm_header.frames=read_word_b(in);
  anm_header.height=getc(in);
  anm_header.width=getc(in);
  anm_header.delay=read_word_b(in);

  for (t=0; t<5; t++) anm_header.unknown[t]=getc(in);

  printf("unanm - Copyright 2004 Michael Kohn\n");
  printf("This program removes frames from Animator V1.0 files\n\n");

  printf("ANM HEADER\n");
  printf("---------------------------\n");
  printf("Signature: %s\n",anm_header.signature);
  printf("Frames: %d\n",anm_header.frames);
  printf("Width: %d\n",anm_header.width);
  printf("Height: %d\n",anm_header.height);
  printf("Delay: %d ms\n",anm_header.delay);
  printf("\n\n");

  max_color=0;

  for (t=0; t<256; t++)
  {
    r=getc(in);
    b=getc(in);
    g=getc(in);

    if (r>max_color) max_color=r;
    if (g>max_color) max_color=g;
    if (b>max_color) max_color=b;

    palette[t]=(r<<16)+(g<<8)+b;
  }

  /* The next few lines make the images brighter.. if needed */

  if (max_color<200)
  {
    i=255/max_color;

    for (t=0; t<256; t++)
    {
      r=(palette[t]>>16)&255;
      g=(palette[t]>>8)&255;
      b=palette[t]&255;

      r=(int)((float)r*i);
      g=(int)((float)g*i);
      b=(int)((float)b*i);

      if (r>255) r=255;
      if (g>255) g=255;
      if (b>255) b=255;

      palette[t]=(r<<16)+(g<<8)+b;
    }
  } 


  width=anm_header.width*2;
  height=anm_header.height*2;

  total_bytes=width*height;

  picture=malloc(total_bytes*4);

  count=0;

  for (r=0; r<anm_header.frames; r++)
  {
    for (y=0; y<height; y=y+2)
    {
      for (x=0; x<width; x=x+2)
      {
        g=getc(in);
        picture[x+(y*width)]=palette[g];
        picture[x+(y*width)+1]=palette[g];
        picture[x+((y+1)*width)]=palette[g];
        picture[x+((y+1)*width)+1]=palette[g];
      }
    }

    sprintf(filename,"frame%d.gif",r);
    out=fopen(filename,"wb");
    if (out==0)
    {
      printf("Couldn't open file for writing: %s\n",filename);
      break;
    }

    lzw_compress(out,picture,width,height,palette);

    fclose(out);
  }

  free(picture);

  fclose(in);

  return 0;
}

